﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace AjpdSoftXSDaXML
{
    public partial class formXSDaXML : Form
    {
        public formXSDaXML()
        {
            InitializeComponent();
        }


        //procedimiento recursivo que convierte un XSD en un XML
        private void XSDtoXML(XmlNode xsdNodo, XmlElement elemento, ref XmlDocument xml)
        {
            if (xsdNodo.HasChildNodes)
            {
                var childs = xsdNodo.ChildNodes;
                foreach (XmlNode node in childs)
                {
                    XmlElement newElement = null;
                    if (node.Name == "xs:element")
                    {
                        newElement = xml.CreateElement(node.Attributes["name"].Value);
                        XSDtoXML(node, newElement, ref xml);
                        if (elemento == null)
                            xml.AppendChild(newElement);
                        else
                            elemento.AppendChild(newElement);
                    }
                    if (node.Name == "xs:attribute")
                    {
                        elemento.SetAttribute(node.Attributes["name"].Value, "");
                    }
                    if ((node.Name == "xs:complexType") || (node.Name == "xs:sequence") || 
                            (node.Name == "xs:schema"))
                        XSDtoXML(node, elemento, ref xml);
                }
            }
        }

        private void btConvertirXSDaXML_Click(object sender, EventArgs e)
        {
            if (!File.Exists(txtFicheroXSD.Text))
            {
                MessageBox.Show("Debe indicar el fichero XSD a convertir a XML.",
                    "Falta fichero XSD origen",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                txtFicheroXSD.Focus();
            }
            else
            {
                if (txtFicheroXML.Text == "")
                {
                    MessageBox.Show("La conversión de XSD a XML se guardará en fichero XML " +
                        "directamente y se mostrará en pantalla. Por lo que debe elegir el " +
                        "fichero XML destino de la conversión.", "Falta fichero XML destino",
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    txtFicheroXML.Focus();
                }
                else
                {
                    try
                    {
                        XmlDocument xsd = new XmlDocument();
                        //xsd.Load(txtFicheroXSD.Text); //para cargar desde fichero directamente
                        xsd.LoadXml(txtXSD.Text); //para cargar desde TextBox
                        XmlNode nodoXSD = xsd.DocumentElement;
                        XmlElement elemento = null;
                        XmlDocument xml = new XmlDocument();
                        XSDtoXML(nodoXSD, elemento, ref xml);

                        //Creamos la declaración XML (cabecera)
                        if (opIncluirCabeceraXML.Checked)
                        {
                            XmlDeclaration cabeceraXML;
                            cabeceraXML = xml.CreateXmlDeclaration(txtVersionXML.Text, 
                                lsCodificacionXML.Text, lsStandalone.Text);
                            //Añadimos un nuevo nodo al documento
                            XmlElement docXMLReferencia = xml.DocumentElement;
                            xml.InsertBefore(cabeceraXML, docXMLReferencia);
                        }

                        //guardamos el fichero XML resultante de la conversión de xsd2xml
                        xml.Save(txtFicheroXML.Text);

                        if (File.Exists(txtFicheroXML.Text))
                        {
                            txtXML.Text = File.ReadAllText(txtFicheroXML.Text);
                            tab.SelectedTab = tabXML;
                            MessageBox.Show("La conversión de XSD a XML se ha realizado correctamente.",
                                "Conversión de XSD a XML finalizada",
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                    }
                    catch(Exception cError)
                    {
                        MessageBox.Show("Se ha producido un error al intentar convertir de XSD a XML:" +
                            Environment.NewLine + Environment.NewLine + cError.Message, 
                            "Error en la conversión de XSD a XML",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void txtFicheroXSD_TextChanged(object sender, EventArgs e)
        {
            if (File.Exists(txtFicheroXSD.Text))
            {
                txtXSD.Text = File.ReadAllText(txtFicheroXSD.Text);
            }
        }

        private void btSelXSD_Click(object sender, EventArgs e)
        {
            OpenFileDialog dSelFichero = new OpenFileDialog();
            dSelFichero.Filter = "XSD (*.xsd)|*.xsd|Todos los archivos (*.*)|*.*";
            dSelFichero.Title = "FIchero XSD a convertir a XML";
            dSelFichero.DefaultExt = "xsd";
            if (dSelFichero.ShowDialog() == DialogResult.OK)
            {
                txtFicheroXSD.Text = dSelFichero.FileName;
            }
        }

        private void btSelXML_Click(object sender, EventArgs e)
        {
            SaveFileDialog dSelFichero = new SaveFileDialog();
            dSelFichero.Filter = "XML (*.xml)|*.xml|Todos los archivos (*.*)|*.*";
            dSelFichero.Title = "FIchero XML destino de XSD";
            dSelFichero.DefaultExt = "xml";
            if (File.Exists(txtFicheroXSD.Text))
            {
                dSelFichero.FileName = Path.ChangeExtension(txtFicheroXSD.Text, "xml");
            }
            dSelFichero.OverwritePrompt = true;
            if (dSelFichero.ShowDialog() == DialogResult.OK)
            {
                txtFicheroXML.Text = dSelFichero.FileName;
            }
        }

        private void btCopiarXMLPortapapeles_Click(object sender, EventArgs e)
        {
            if (txtXML.Text != "")
            {
                try
                {
                    Clipboard.SetDataObject(txtXML.Text);
                    MessageBox.Show("Datos XML copiados al portapapeles correctamente.", "Portapapeles",
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception cError)
                {
                    MessageBox.Show("Error al copiar datos XML al portapapeles: " +
                        Environment.NewLine + Environment.NewLine + cError.Message,
                        "Portapapeles", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
                MessageBox.Show("No hay datos XML para copiar al portapapeles.", "Portapapeles",
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.ajpdsoft.com");
        }
    }
}
